package asteroids;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.util.Random;

public class AsteroidExtended extends Asteroid {

	private Position direction;
	private float speed;

	private Circle collisionCircle;

	private static final float sizeMultiplier = 20f;
	private static final Random rand = new Random();
	
	private static int[] shipX = new int[]{5, 10, 5,  -5, -10, -5};
    private static int[] shipY = new int[]{8, 0,  -8, -8, 0,    8};
    
    private static final Polygon poly = new Polygon(shipX, shipY, shipX.length);

	public AsteroidExtended() {
		super(rand.nextFloat() * AsteroidsGame.screenX, rand.nextFloat() * AsteroidsGame.screenY, 3);
		
		speed = rand.nextFloat() * speedLimit;
		float xDir = rand.nextFloat() - 0.5f;
		float yDir = rand.nextFloat() - 0.5f;
		direction = new Position(xDir, yDir);
		
		collisionCircle = new Circle(size * sizeMultiplier, position);
	}

	public AsteroidExtended(float x, float y, int s) {
		this(new Position(x, y), s);
	}

	public AsteroidExtended(Position p, int s) {
		super(p, s);
		collisionCircle = new Circle(size * sizeMultiplier, this.position);
		
		speed = rand.nextFloat() * speedLimit;
		float xDir = rand.nextFloat() - 0.5f;
		float yDir = rand.nextFloat() - 0.5f;
		direction = new Position(xDir, yDir);
	}

	public void advance() {
		move(direction.getX() * speed, direction.getY() * speed);
	}
	
	public void replaceIfNeeded() {
		if(position.getX() < 0) {
			position.translate(AsteroidsGame.screenX, 0);
		} else if(position.getY() < 0) {
			position.translate(0, AsteroidsGame.screenY);
		} else if(AsteroidsGame.screenX < position.getX()) {
			position.translate(-1 * AsteroidsGame.screenX, 0);
		} else if(AsteroidsGame.screenY < position.getY()) {
			position.translate(0, -1 * AsteroidsGame.screenY);
		}
	}

	public boolean collideWithBullet(Bullet b) {
		return b.collideWithAsteroid(collisionCircle);
	}
	
	public void draw(Graphics2D g) {
		Graphics2D g2d = (Graphics2D) g.create();
		g2d.setColor(Color.GRAY);
		
		g2d.translate(position.getX(), position.getY());
		g2d.scale(size, size);
		
		g2d.draw(poly);
		g2d.fill(poly);
		
		g2d.dispose();
	}
}
