package asteroids;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.Timer;

public class MainLoop extends JPanel implements ActionListener {

	private Timer timer;
	private SpaceShipExtended player;
	private InputManager inputManager;
	
	private List<Bullet> bulletContainer; 
	private List<AsteroidExtended> asteroidContainer;

	public MainLoop(AsteroidsGame parent) {

		bulletContainer = new ArrayList<Bullet>(10);
		asteroidContainer = new ArrayList<AsteroidExtended>(100);
		
		initBoard(parent);
	}

	private void initBoard(AsteroidsGame parent) {

		setFocusable(true);
		player = new SpaceShipExtended(400, 300, bulletContainer);
		timer = new Timer(50, this);
		timer.start();
		
		inputManager = new InputManager(player);
		
		addKeyListener(inputManager);
	}

	private void createAsteroids() {
		for(int i = 0; i < 10; i++) {
			asteroidContainer.add(new AsteroidExtended());
		}
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		
		moveSpaceShip();
		moveAsteroids();
		moveBullets();
		handleCollisions();
		inputManager.handleDownKeys();

		repaint();
	}

	private void moveSpaceShip() {
		player.replaceIfNeeded();
	}

	public void start() {

		createAsteroids();

		timer.start();
	}

	private void moveAsteroids() {
		for(AsteroidExtended a : asteroidContainer) {
			a.advance();
			a.replaceIfNeeded();
		}
	}
	
	private void moveBullets() {
		for(Bullet b : bulletContainer) {
			b.advance();
		}
		for(int i = 0; i < bulletContainer.size(); i++) {
			if(bulletContainer.get(i).isOut()) {
				bulletContainer.remove(i);
				i--;
			}
		}
	}

	private void handleCollisions() {
		for(int j = 0; j < bulletContainer.size(); j++) {
			for(int i = 0; i < asteroidContainer.size(); i++) {
				AsteroidExtended a = asteroidContainer.get(i);
				Bullet b = bulletContainer.get(j);
				if(a.collideWithBullet(b)) {
					asteroidContainer.remove(i);
					bulletContainer.remove(j);
					asteroidBulletCollision(a, b);
					j--;
					break;
				}
			}
		}
	}
	
	private void asteroidBulletCollision(AsteroidExtended a, Bullet b) {
		AsteroidExtended[] newAsteroids = a.destroy();
		for(AsteroidExtended ast : newAsteroids) {
			asteroidContainer.add(ast);
		}
	}

	private void doDrawing(Graphics2D g) {
		player.draw(g);
		
		for(Bullet b : bulletContainer) {
			b.draw(g);
		}
		
		for(AsteroidExtended a : asteroidContainer) {
			a.draw(g);
		}
	}

	@Override
	public void paintComponent(Graphics g) {

		super.paintComponent(g);
		doDrawing((Graphics2D)g);
	}
}
